@@ -1,6 +1,9 @@
-from aiohttp import web
+import urllib
 
-known_token = set()
+from aiohttp import web
+from aiohttp_middlewares import cors_middleware  # type: ignore
+
+from anyscale import ANYSCALE_ENV
 
 
 async def error_page(request: web.Request) -> web.Response:
@@ -38,6 +41,10 @@ async def error_page(request: web.Request) -> web.Response:
 async def index(request: web.Request) -> web.Response:
     token = request.query.get("token")
     redirect_to = request.query.get("redirect_to")
+    # Sometimes we don't want to perform the redirect.
+    # We just want to authenticate and download the cookie,
+    # for example in the case of the WebTerminal.
+    auth_only = request.query.get("auth_only")
 
     path = {
         "tensorboard": "/tensorboard/",
@@ -46,6 +53,7 @@ async def index(request: web.Request) -> web.Response:
         "hosted_dashboard": "/metrics/redirect",
         "webterminal": "/webterminal/",
         "anyscaled": "/anyscaled/",
+        "metrics": "/metrics",
     }
 
     if not token or not redirect_to or redirect_to not in path.keys():
@@ -55,10 +63,13 @@ async def index(request: web.Request) -> web.Response:
             "redirect_to={tensorboard, dashboard, grafana, webterminal}.",
             status=401,
         )
-    # TODO(simon): set token on startup
-    known_token.add(token)
 
-    resp = web.HTTPFound(path[redirect_to])
+    resp = (
+        web.HTTPFound(path[redirect_to])
+        if auth_only is None
+        else web.Response(text="ok")
+    )
+    is_production = ANYSCALE_ENV["ANYSCALE_HOST"] == "https://beta.anyscale.com"
     if redirect_to == "webterminal":
         # The default for samesite is None currently but it is migrating
         # to Lax for most browser. Explicitly setting it here will
@@ -66,7 +77,7 @@ async def index(request: web.Request) -> web.Response:
         # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie/SameSite
         resp.set_cookie("anyscale-token", token, secure=True, samesite="None")
     else:
-        resp.set_cookie("anyscale-token", token, secure=True)
+        resp.set_cookie("anyscale-token", token, secure=is_production)
     return resp
 
 
@@ -76,20 +87,47 @@ async def authorize(request: web.Request) -> web.Response:
         request.headers.get("X-Forwarded-Uri", "unknown"),
     )
     cookies = request.cookies
-    if cookies.get("anyscale-token") in known_token:
+    if cookies.get("anyscale-token") == request.config_dict["auth_token"]:
         return web.Response(text="Authorized", status=200)
     else:
+        # Prometheus does not set a cookie
+        if request.headers.get("X-Forwarded-Uri") == "/metrics":
+            ref = request.headers.get("Referer", "")
+            token_list = urllib.parse.parse_qs(urllib.parse.urlsplit(ref).query).get(
+                "token", []
+            )
+            if (
+                len(token_list) == 1
+                and token_list[0] == request.config_dict["auth_token"]
+            ):
+                return web.Response(text="Authorized", status=200)
         return web.Response(text="Unauthorized", status=401)
 
 
-auth_app = web.Application()
-auth_app.add_routes(
-    [
-        web.get("/", index),
-        web.get("/authorize", authorize),
-        web.get("/error_page", error_page),
-    ]
-)
+def make_auth_proxy_app(auth_token: str) -> web.Application:
+    auth_app = web.Application()
+    auth_app.add_routes(
+        [
+            web.get("/", index),
+            web.get("/authorize", authorize),
+            web.get("/error_page", error_page),
+        ]
+    )
 
-app = web.Application()
-app.add_subapp("/auth", auth_app)
+    origins = []
+    # ANYSCALE_ENV is only a dictionary values when
+    # this server runs on the head node.
+    # If its a dictionary, use the ANYSCALE_HOST as whitelisted host
+    # these values could be an IP address in developer instances
+    # or beta.anyscale.com in production
+    if "ANYSCALE_HOST" in ANYSCALE_ENV:
+        origins = [ANYSCALE_ENV["ANYSCALE_HOST"]]
+
+    app = web.Application(
+        middlewares=[cors_middleware(origins=origins, allow_credentials=True)]
+    )
+
+    app.add_subapp("/auth", auth_app)
+    app["auth_token"] = auth_token
+
+    return app
