@@ -5,14 +5,18 @@ import logging
 from types import ModuleType
 from typing import Any, cast, Dict, List, Optional
 
+from openapi_client.rest import ApiException  # type: ignore
+from ray.autoscaler._private.aws.node_provider import to_aws_format
 from ray.autoscaler._private.command_runner import DockerCommandRunner, SSHCommandRunner
+from ray.autoscaler._private.util import hash_launch_conf
 from ray.autoscaler.command_runner import CommandRunnerInterface
+from ray.autoscaler.tags import TAG_RAY_CLUSTER_NAME
 
 # This try block exists because
 # 1. This module is passed to the autoscaler on the head node as an external node provider
 # 2. Import paths are different for different versions of ray
 try:
-    from ray.autoscaler._private.aws.node_provider import NodeProvider
+    from ray.autoscaler.node_provider import NodeProvider
     from ray.autoscaler._private.providers import (
         _get_node_provider,
         _NODE_PROVIDERS,
@@ -25,6 +29,7 @@ except ModuleNotFoundError:
     )
 
 from anyscale.api import instantiate_api_client
+from anyscale.client.openapi_client.models.add_instance_pool_member import AddInstancePoolMember  # type: ignore
 from anyscale.client.openapi_client.models.create_nodes_options import (  # type: ignore
     CreateNodesOptions,
 )
@@ -32,9 +37,12 @@ from anyscale.client.openapi_client.models.nodes_options import NodesOptions  #
 from anyscale.client.openapi_client.models.non_terminated_nodes_options import (  # type: ignore
     NonTerminatedNodesOptions,
 )
+from anyscale.client.openapi_client.models.query_pool_size import QueryPoolSize  # type: ignore
+from anyscale.client.openapi_client.models.request_instance_pool_member import RequestInstancePoolMember  # type: ignore
 from anyscale.client.openapi_client.models.set_node_tags_options import (  # type: ignore
     SetNodeTagsOptions,
 )
+from anyscale.util import format_api_exception
 
 logger = logging.getLogger(__name__)
 
@@ -42,11 +50,12 @@ logger = logging.getLogger(__name__)
 class AnyscalePoolingNodeProvider:
     """NodeProvider wrapper for injecting product-specific functionality.
 
-    Currently a no-op wrapper,
-    but we plan on adding instance pool functionality here soon:
-        https://docs.google.com/document/d/15goOexCiGkbzz7tUbILMc10Gmjf86PiyCtIl75-xGUw/edit
+    https://docs.google.com/document/d/15goOexCiGkbzz7tUbILMc10Gmjf86PiyCtIl75-xGUw/edit
     """
 
+    POOLED_INSTANCE_TAG = "anyscale-launched-from-pool"
+    INSTANCE_REUSE_TAG = "anyscale-instance-reuse-id"
+
     def __init__(self, provider_config: Dict[str, Any], cluster_name: str) -> None:
         """Implements manual inheritance from NodeProvider.
 
@@ -63,9 +72,34 @@ class AnyscalePoolingNodeProvider:
             of the original NodeProvider class which we are wrapping around.
         """
 
-        inner_provider_config = provider_config["inner_provider"]
+        self.provider_config = provider_config
+        self.inner_provider_config = provider_config["inner_provider"]
         # This is the actual NodeProvider object we are wrapping around.
-        self.inner_provider = _get_node_provider(inner_provider_config, cluster_name)
+        self.inner_provider = _get_node_provider(
+            self.inner_provider_config, cluster_name
+        )
+
+        self.cloud_id = self._get_from_configs("cloud_id")
+        self.api_client = instantiate_api_client(
+            cli_token=self._get_from_configs("cli_token"),
+            host=self._get_from_configs("host"),
+        )
+
+    def __getattr__(self, name: str) -> Any:
+        """Implements inheritance from self.inner_provider."""
+        return getattr(self.inner_provider, name)
+
+    def _get_from_configs(self, key: str, default: Any = KeyError) -> Any:
+        """Searches both the main and inner provider configs for a key."""
+
+        if key in self.provider_config:
+            return self.provider_config[key]
+        elif key in self.inner_provider_config:
+            return self.inner_provider_config[key]
+        elif default == KeyError:
+            raise KeyError
+        else:
+            return default
 
     @staticmethod
     def _get_inner_provider_class_from_config(cluster_config: Dict[str, Any]) -> Any:
@@ -85,6 +119,8 @@ class AnyscalePoolingNodeProvider:
 
     @staticmethod
     def bootstrap_config(cluster_config: Dict[str, Any]) -> Dict[str, Any]:
+        cluster_config = copy.deepcopy(cluster_config)
+
         # Get the inner_provider class for the static method.
         inner_provider_cls = AnyscalePoolingNodeProvider._get_inner_provider_class_from_config(
             cluster_config
@@ -109,10 +145,14 @@ class AnyscalePoolingNodeProvider:
     def fillout_available_node_types_resources(
         cluster_config: Dict[str, Any]
     ) -> Dict[str, Any]:
+        cluster_config = copy.deepcopy(cluster_config)
+
         inner_provider_cls = AnyscalePoolingNodeProvider._get_inner_provider_class_from_config(
             cluster_config
         )
 
+        # Temporarily promote the inner provider to the outer provider
+        # so we can configure it.
         primary_provider = cluster_config["provider"]
         cluster_config["provider"] = cluster_config["provider"]["inner_provider"]
 
@@ -120,13 +160,202 @@ class AnyscalePoolingNodeProvider:
             cluster_config
         )
 
+        # Restore the original outer and inner providers.
         primary_provider["inner_provider"] = cluster_config["provider"]
         cluster_config["provider"] = primary_provider
+
         return cluster_config
 
-    def __getattr__(self, name: str) -> Any:
-        """Implements inheritance from self.inner_provider."""
-        return getattr(self.inner_provider, name)
+    def non_terminated_nodes(self, tag_filters: Dict[str, str]) -> List[str]:
+        logger.warn("non_terminated_nodes called with {}".format(tag_filters))
+        res: List[str] = self.inner_provider.non_terminated_nodes(tag_filters)
+        return res
+
+    def create_node(
+        self, node_config: Dict[str, Any], tags: Dict[str, str], count: int
+    ) -> None:
+
+        # Calculate the instance reuse ID straight away
+        # so that it can be stored in the node's tags
+        # for later potential insertion into the pool.
+        #
+        # The reuse ID is currently the hash of the node config, minus user tags.
+        config_to_hash = copy.deepcopy(node_config)
+        if "TagSpecifications" in config_to_hash:
+            del config_to_hash["TagSpecifications"]
+        instance_reuse_id = hash_launch_conf(config_to_hash, dict())
+        tags[self.INSTANCE_REUSE_TAG] = instance_reuse_id
+
+        # Only handle AWS for now.
+        # TODO(xcl): add support for non-aws instances.
+        bypass_rapid_start = self._get_from_configs("bypass_rapid_start", None)
+        pool_disabled = self._pool_limit() == 0
+        # Pooling version (with checks).
+        if not (bypass_rapid_start or pool_disabled):
+            if self._is_aws_provider():
+                for i in range(count):
+                    self._create_single_node_aws(node_config, tags)
+                return
+
+        # Non-pooling version.
+        logger.info("Configuration bypasses Rapid Start; launching node normally.")
+        self.inner_provider.create_node(node_config, tags, count)
+
+    def _create_single_node_aws(
+        self, node_config: Dict[str, Any], tags: Dict[str, str],
+    ) -> None:
+        """create_node implementation for when inner provider is AWS."""
+
+        instance_pool_request = RequestInstancePoolMember(
+            cloud_id=self.cloud_id, instance_reuse_id=tags[self.INSTANCE_REUSE_TAG],
+        )
+
+        try:
+            instance_pool_member = self.api_client.request_instance_from_pool_api_v2_instancepools_request_instance_post(
+                instance_pool_request
+            ).result
+            node_id = instance_pool_member.instance_id
+        except ApiException:
+            # We expect to get this exception if there is no instance
+            # that matches this request.
+            node_id = None
+
+        # If we get a node_id back from the instance pool,
+        # start this node.
+        if node_id is not None:
+            node = self.inner_provider._get_cached_node(node_id)
+            node.wait_until_stopped()
+            logger.info("Launching node from instance pool! ID {}".format(node_id))
+            self.inner_provider.ec2.meta.client.start_instances(InstanceIds=[node_id])
+
+            tags[self.POOLED_INSTANCE_TAG] = "true"
+
+            # Various mandatory tag modification logic
+            # that happens in OSS under create_node.
+            tags = to_aws_format(tags)
+            tags[TAG_RAY_CLUSTER_NAME] = self.cluster_name
+            # Add the tags from the node config's TagSpecifications.
+            if "TagSpecifications" in node_config:
+                for tagspec in node_config["TagSpecifications"]:
+                    if tagspec["ResourceType"] == "instance":
+                        for tagpair in tagspec["Tags"]:
+                            tags[tagpair["Key"]] = tagpair["Value"]
+
+            logger.info("Setting node tags on reused instance: {}".format(str(tags)))
+            self.set_node_tags(node_id, tags)
+
+        # Otherwise, start a node normally.
+        else:
+            logger.info("Instance pool empty; launching node from EC2.")
+            self.inner_provider.create_node(node_config, tags, count=1)
+
+    def terminate_node(self, node_id: str) -> None:
+        # Only handle AWS for now.
+        # TODO(xcl): add support for non-aws instances.
+        tags = self.node_tags(node_id)
+        launch_hash = tags.get(self.INSTANCE_REUSE_TAG)
+
+        # Check prerequisites and fallback to normal termination
+        # if prerequisites are not met.
+        if launch_hash is None:
+            logger.warn(
+                "No instance reuse tag found, so node is ineligible for pool. "
+                "This should only happen if the node was launched using an older CLI version. "
+            )
+            self.inner_provider.terminate_node(node_id)
+            return
+
+        if not self._is_aws_provider():
+            logger.warn(
+                "Instance pool not yet supported for this cloud provider; node will not be added to pool."
+            )
+            self.inner_provider.terminate_node(node_id)
+            return
+
+        if self._room_in_pool() <= 0:
+            logger.warn("No room in pool; node will not be added to pool.")
+            self.inner_provider.terminate_node(node_id)
+            return
+
+        # Prerequisites are met here;
+        # proceed with putting node in instance pool.
+
+        node = self.inner_provider._get_cached_node(node_id)
+        self._prepare_node_for_idempotent_reuse(node)
+        # TODO(xcl): Determine which tags should be cleared vs left uncleared for correctness here.
+
+        node.stop()
+
+        logger.info("Sending node {} to the instance pool.".format(node_id))
+
+        instance_reuse_id = launch_hash
+        instance_pool_submission = AddInstancePoolMember(
+            instance_id=node_id,
+            cloud_id=self.cloud_id,
+            instance_reuse_id=instance_reuse_id,
+        )
+        with format_api_exception(ApiException):
+            self.api_client.add_instance_to_pool_api_v2_instancepools_add_instance_post(
+                instance_pool_submission
+            )
+
+    def _prepare_node_for_idempotent_reuse(self, node: Any) -> None:
+        """Does necessary resets on node to prepare it for reuse."""
+
+        # Currently, we will explicitly not do anything.
+        # The only potential issue right now is file mounts from previous sessions,
+        # which should ideally be garbage collected (TODO (xcl)),
+        # but this is not a high priority because we may also just
+        # expire sessions in the pool after they've been reused a lot.
+        pass
+
+    def terminate_nodes(self, node_ids: List[str]) -> None:
+        # TODO(xcl): add support for non-aws instances.
+        if self._is_aws_provider():
+            for node_id in node_ids:
+                self.terminate_node(node_id)
+        else:
+            self.inner_provider.terminate_nodes(node_ids)
+
+    def _is_aws_provider(self) -> bool:
+        """Helper method; whether the inner provider is AWS or not."""
+
+        # 1. The inner provider can be AWS directly.
+        if self.inner_provider_config["type"] == "aws":
+            return True
+
+        # 2. If the inner provider is of type "external",
+        # it can still represent an AWS provider,
+        # only if the corresponding module is our AnyscaleAWSNodeProvider.
+        if (
+            self.inner_provider_config["type"] == "external"
+            and self.inner_provider_config["module"]
+            == "anyscale.autoscaler.aws.node_provider.AnyscaleAWSNodeProvider"
+        ):
+            return True
+
+        return False
+
+    def _pool_size(self) -> int:
+        """Get this cloud's current pool size."""
+        query = QueryPoolSize(cloud_id=self.cloud_id)
+        with format_api_exception(ApiException):
+            pool_size = self.api_client.pool_size_api_v2_instancepools_pool_size_post(
+                query
+            )
+        return int(pool_size)
+
+    def _pool_limit(self) -> int:
+        """Get this cloud's pool size limit."""
+
+        pool_limit = self.api_client.get_cloud_api_v2_clouds_cloud_id_get(
+            cloud_id=self.cloud_id
+        ).result.config.max_stopped_instances
+
+        return int(pool_limit)  # already int; just cast for typing
+
+    def _room_in_pool(self) -> int:
+        return self._pool_limit() - self._pool_size()
 
 
 class AnyscaleInstanceManagerNodeProvider:
@@ -164,6 +393,7 @@ class AnyscaleInstanceManagerNodeProvider:
 
     @staticmethod
     def bootstrap_config(cluster_config: Dict[str, Any]) -> Dict[str, Any]:
+        cluster_config = copy.deepcopy(cluster_config)
         inner_provider_cls = AnyscalePoolingNodeProvider._get_inner_provider_class_from_config(
             cluster_config
         )
@@ -180,6 +410,7 @@ class AnyscaleInstanceManagerNodeProvider:
     def fillout_available_node_types_resources(
         cluster_config: Dict[str, Any]
     ) -> Dict[str, Any]:
+        cluster_config = copy.deepcopy(cluster_config)
         inner_provider_cls = AnyscalePoolingNodeProvider._get_inner_provider_class_from_config(
             cluster_config
         )
