@@ -2,23 +2,25 @@
 Placed in frontend/cli/anyscale package to be in the PYTHONPATH when
 autoscaler searches for the module.
 """
-import json
 import logging
 import subprocess
 import sys
-import time
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List
 import uuid
 
 import ray
 from ray.autoscaler.node_provider import NodeProvider
-import yaml
 
 from anyscale.util import send_json_request
 from backend.server.services.gateway_commandrunner_service import (
     GatewaySSHCommandRunner,
 )
-from backend.server.services.gateway_router_service import GatewayID, GatewayRequest
+from backend.server.services.gateway_router_service import (
+    CloudGatewayException,
+    CloudGatewayExceptionString,
+    GatewayID,
+    GatewayRequest,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -40,7 +42,7 @@ class ServerNodeProvider(NodeProvider):  # type: ignore
     def __init__(
         self, provider_config: Dict[str, Any], cluster_name: str,
     ):
-        self.gateway_id = provider_config["gateway_id"]
+        self.gateway_id = str(provider_config["gateway_id"])
         self.provider_config = provider_config["subprovider"]
         self.cluster_name = cluster_name
         # TODO(ameer): uncomment this to validate user registered to this gateway
@@ -74,8 +76,12 @@ class ServerNodeProvider(NodeProvider):  # type: ignore
         ray_actor_handler = ray.get_actor("GatewayRouterActor")
         ray.get(ray_actor_handler.push_request.remote(self.gateway_id, request))
         response = ray.get(
-            ray_actor_handler.pull_response.remote(self.gateway_id, request)
+            ray_actor_handler.pull_response.remote(
+                self.gateway_id, request["request_id"]
+            )
         )
+        if response["data"] == CloudGatewayExceptionString:
+            raise CloudGatewayException()
         return response["data"]
 
     def non_terminated_nodes(self, tag_filters: Dict[str, Any]) -> Any:
@@ -121,7 +127,7 @@ class ServerNodeProvider(NodeProvider):  # type: ignore
         request = {"type": "terminate_node", "args": (node_id,)}
         self.get_response(request)
 
-    def terminate_nodes(self, node_ids: str) -> None:
+    def terminate_nodes(self, node_ids: List[str]) -> None:
         request = {"type": "terminate_nodes", "args": (node_ids,)}
         self.get_response(request)
 
@@ -160,3 +166,13 @@ class ServerNodeProvider(NodeProvider):  # type: ignore
         node_provider = ServerNodeProvider(config["provider"], config["cluster_name"])
         bootstrapped_config: Dict[str, Any] = node_provider.get_response(request)
         return bootstrapped_config
+
+    def prepare_for_head_node(self, cluster_config: Dict[str, Any]) -> Dict[str, Any]:
+        # The ssh key is copied from ~/.ssh/id_rsa to ~/ray_bootstrap_key.pem using custom
+        # commands in the cluster yaml.
+        # After runnning these commands, ~/ray_bootstrap_key.pem becomes available on the head
+        # node and we add it in the line below.
+        cluster_config["auth"]["ssh_private_key"] = "~/ray_bootstrap_key.pem"
+        # We also update the head node cluster config with the real provider that it should use.
+        cluster_config["provider"] = cluster_config["provider"]["subprovider"]
+        return cluster_config
