@@ -128,6 +128,33 @@ def _update_file_mounts(
     return cluster_config
 
 
+@_with_safe_cluster_config  # type: ignore
+def _configure_min_max_workers(
+    cluster_config: Dict[str, Any],
+    min_workers: Optional[int] = None,
+    max_workers: Optional[int] = None,
+) -> Dict[str, Any]:
+    if min_workers and "min_workers" in cluster_config:
+        cluster_config["min_workers"] = min_workers
+    if max_workers and "max_workers" in cluster_config:
+        cluster_config["max_workers"] = max_workers
+    if "available_node_types" in cluster_config:
+        for node in cluster_config["available_node_types"]:
+            if min_workers and (
+                "min_workers" in cluster_config["available_node_types"][node]
+            ):
+                cluster_config["available_node_types"][node][
+                    "min_workers"
+                ] = min_workers
+            if max_workers and (
+                "max_workers" in cluster_config["available_node_types"][node]
+            ):
+                cluster_config["available_node_types"][node][
+                    "max_workers"
+                ] = max_workers
+    return cluster_config
+
+
 def configure_for_session(
     session_id: str,
     project_root_dir: str,
