@@ -1,10 +1,11 @@
 import datetime
 import logging
+import os
 from typing import Dict, List, Optional
 
 import click
 
-from anyscale.auth_proxy import app as auth_proxy_app
+from anyscale.auth_proxy import make_auth_proxy_app
 import anyscale.legacy_projects as ray_scripts
 from anyscale.project import (
     get_project_id,
@@ -133,9 +134,19 @@ def session_finish_command(command_id: str, stop: bool, terminate: bool) -> None
 
 # Note: These functions are not normally updated, please cc yiran or ijrsvt on changes.
 @session_cli.command(name="auth_start", help="Start the auth proxy", hidden=True)
-def auth_start() -> None:
+@click.option(
+    "--auth-token", help="Token for authentication proxy", type=str, required=False
+)
+def auth_start(auth_token: Optional[str]) -> None:
     from aiohttp import web
 
+    # TODO(pcm): Make this required and pass in after the Docker image is migrated.
+    if not auth_token:
+        auth_token = os.environ["ANYSCALE_AUTH_TOKEN"]
+
+    assert auth_token, "Auth Token must be set by parameter or environment variable."
+
+    auth_proxy_app = make_auth_proxy_app(auth_token)
     web.run_app(auth_proxy_app)
 
 
@@ -151,10 +162,34 @@ def auth_start() -> None:
 @click.option(
     "--use-debugger", help="Activate the Anyscale debugger.", is_flag=True,
 )
-def web_terminal_server(deploy_environment: str, use_debugger: bool) -> None:
+@click.option(
+    "--cli-token",
+    help="Anyscale cli token used to instantiate anyscale openapi",
+    type=str,
+    required=True,
+)
+@click.option(
+    "--host",
+    help="Anyscale host used to instantiate anyscale openapi (beta.anyscale.com for example)",
+    type=str,
+    required=True,
+)
+@click.option(
+    "--working-dir",
+    help="The working directory for this anyscale session. The webterminal will be opened from this directory.",
+    type=str,
+    required=True,
+)
+def web_terminal_server(
+    deploy_environment: str,
+    use_debugger: bool,
+    cli_token: str,
+    host: str,
+    working_dir: str,
+) -> None:
     from anyscale.webterminal.webterminal import main
 
-    main(deploy_environment, use_debugger)
+    main(deploy_environment, use_debugger, cli_token, host, working_dir)
 
 
 @click.command(
