@@ -107,15 +107,12 @@ class ListController(object):
             table = []
             for session in sessions:
                 created_at = humanize_timestamp(session.created_at)
-                if not session.snapshots_history:
-                    session.snapshots_history.append("N/A")
                 record = [
                     session.name,
                     " {}".format(
                         self.get_session_status_and_commands(session=session)[0]
                     ),
                     created_at,
-                    session.snapshots_history[0],
                 ]
                 if show_all:
                     table.append(
@@ -137,14 +134,12 @@ class ListController(object):
                     table.append(record)
             if not show_all:
                 table_string = tabulate.tabulate(
-                    table,
-                    headers=["SESSION", "STATUS", "CREATED", "SNAPSHOT"],
-                    tablefmt="plain",
+                    table, headers=["SESSION", "STATUS", "CREATED"], tablefmt="plain",
                 )
             else:
                 table_string = tabulate.tabulate(
                     table,
-                    headers=["ACTIVE", "SESSION", "STATUS", "CREATED", "SNAPSHOT"],
+                    headers=["ACTIVE", "SESSION", "STATUS", "CREATED"],
                     tablefmt="plain",
                 )
 
