@@ -21,7 +21,11 @@ from anyscale.autosync_heartbeat import (
 from anyscale.client.openapi_client import CreateSessionFromSnapshotOptions, Session, SessionFinishUpOptions, SessionState, SessionUpOptions, SetupInitializeSessionOptions, StopSessionOptions  # type: ignore
 from anyscale.client.openapi_client.api.default_api import DefaultApi  # type: ignore
 from anyscale.cloud import get_cloud_id_and_name
-from anyscale.cluster_config import configure_for_session, get_cluster_config
+from anyscale.cluster_config import (
+    _configure_min_max_workers,
+    configure_for_session,
+    get_cluster_config,
+)
 from anyscale.project import (
     get_project_id,
     get_project_session,
@@ -39,6 +43,7 @@ from anyscale.util import (
     populate_session_args,
     slugify,
     validate_cluster_configuration,
+    wait_for_head_node_start,
     wait_for_session_start,
 )
 from anyscale.utils.aws_credentials_util import (
@@ -54,7 +59,7 @@ def get_head_node_ip(cluster_config: Union[Dict[str, Any], str]) -> Any:
 
 
 def rsync(*args: Any, **kwargs: Any) -> None:
-    from ray.autoscaler._private.commands import rsync as ray_rsync
+    from ray.autoscaler.sdk import rsync as ray_rsync
 
     ray_rsync(*args, **kwargs)
 
@@ -72,6 +77,7 @@ class SessionController:
         delete: bool,
         workers_only: bool,
         keep_min_workers: bool,
+        skip_snapshot: bool = False,
     ) -> None:
         project_definition = load_project_or_throw()
         project_id = get_project_id(project_definition.root)
@@ -95,6 +101,7 @@ class SessionController:
                     workers_only=workers_only,
                     keep_min_workers=keep_min_workers,
                     delete=delete,
+                    take_snapshot=not skip_snapshot,
                 ),
             )
 
@@ -103,10 +110,44 @@ class SessionController:
         url = get_endpoint(f"/projects/{project_id}")
         print(f"Session {session_names_str} stopping. View progress at {url}")
 
+    def _load_cluster_config(
+        self, config: Optional[str], session_exists: bool, project_definition: Any
+    ) -> Optional[Dict[str, Any]]:
+        """config option to cluster config.
+
+        Excision of existing logic for C901.
+        """
+        cluster_config = None
+        if not session_exists:
+            config_attribute_provided = True
+            if not config:
+                config_attribute_provided = False
+                config = project_definition.cluster_yaml()
+
+            config = str(config)  # for typing.
+            if not os.path.exists(config):
+                if config_attribute_provided:
+                    raise ValueError("Config file {} not found".format(config))
+                else:
+                    raise ValueError(
+                        "No config param provided and default config file session-default.yaml not found. Please provide a config file using the --config attribute."
+                    )
+
+        if config:
+            with open(config) as f:
+                cluster_config_filled = populate_session_args(f.read(), config)
+                cluster_config = yaml.safe_load(cluster_config_filled)
+
+            validate_cluster_configuration(config, cluster_config, self.api_client)
+
+        return cluster_config
+
     def up(
         self,
         session_name: Optional[str],
         config: Optional[str],
+        build_id: Optional[str],
+        compute_template_id: Optional[str],
         min_workers: Optional[int],
         max_workers: Optional[int],
         no_restart: bool,
@@ -114,12 +155,15 @@ class SessionController:
         disable_sync: bool,
         cloud_id: Optional[str],
         cloud_name: Optional[str],
+        idle_timeout: Optional[int],
         yes: bool,
         verbose: bool = False,
+        no_rapid_start: bool = False,
     ) -> None:
         import ray.scripts.scripts as autoscaler_scripts
 
         # TODO(ilr) Split this into ~3 functions, look at jbai's comments on #1469
+
         project_definition = load_project_or_throw()
         project_id = get_project_id(project_definition.root)
 
@@ -136,28 +180,41 @@ class SessionController:
 
         session_exists = len(session_list) > 0
 
-        if not session_exists:
-            config_attribute_provided = True
-            if not config:
-                config_attribute_provided = False
-                config = project_definition.cluster_yaml()
+        # Deduce whether we are launching this session
+        # from a cluster config yaml
+        # as opposed to a build ID + compute template ID combination.
+        # The logic is:
+        #  - If either build or compute template is specified,
+        #    we will use those. Both must be specified.
+        #  - Otherwise, we will use a cluster config yaml.
+        #     - If a yaml name is explicitly specified, we will use that.
+        #     - Otherwise, look for session-default.yaml.
+        using_cluster_config = build_id is None and compute_template_id is None
+        if not using_cluster_config:
+            if config:
+                raise ValueError(
+                    "If launching from a build/compute template, a config file cannot be specified."
+                )
+            if not build_id or not compute_template_id:
+                raise ValueError(
+                    "Both build ID and Compute Template must be specified if launching a session that way."
+                )
 
-            if not os.path.exists(config):
-                if config_attribute_provided:
-                    raise ValueError("Config file {} not found".format(config))
-                else:
-                    raise ValueError(
-                        "No config param provided and default config file session-default.yaml not found. Please provide a config file using the --config attribute."
-                    )
+        # If using a user cluster config, load it.
+        maybe_cluster_config = None
+        if using_cluster_config:
+            maybe_cluster_config = self._load_cluster_config(
+                config=config,
+                session_exists=session_exists,
+                project_definition=project_definition,
+            )
 
-        cluster_config = None
-
-        if config:
-            with open(config) as f:
-                cluster_config_filled = populate_session_args(f.read(), config)
-                cluster_config = yaml.safe_load(cluster_config_filled)
-
-            validate_cluster_configuration(config, cluster_config, self.api_client)
+        if no_rapid_start:
+            if not maybe_cluster_config:
+                raise ValueError(
+                    "Please specify a cluster config if using no_rapid_start."
+                )
+            maybe_cluster_config["provider"]["bypass_rapid_start"] = True
 
         cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_id, cloud_name)
         assert cloud_id is not None, "Failed to get cloud."
@@ -167,10 +224,13 @@ class SessionController:
                 session_up_options=SessionUpOptions(
                     project_id=project_id,
                     name=session_name,
-                    cluster_config={"config": json.dumps(cluster_config)}
-                    if cluster_config
+                    cluster_config={"config": json.dumps(maybe_cluster_config)}
+                    if maybe_cluster_config
                     else None,
+                    build_id=build_id,
+                    compute_template_id=compute_template_id,
                     cloud_id=cloud_id,
+                    idle_timeout=idle_timeout,
                 )
             ).result
         session_id = up_response.session_id
@@ -275,6 +335,8 @@ class SessionController:
         self,
         session_name: Optional[str],
         config: Optional[str],
+        build_id: Optional[str],
+        compute_template_id: Optional[str],
         min_workers: Optional[int],
         max_workers: Optional[int],
         no_restart: bool,
@@ -282,13 +344,18 @@ class SessionController:
         disable_sync: bool,
         cloud_id: Optional[str],
         cloud_name: Optional[str],
-        yes: bool,
+        idle_timeout: Optional[int],
         verbose: bool = False,
     ) -> None:
         # TODO(ilr) Split this into ~3 functions, look at jbai's comments on #1469
         project_definition = load_project_or_throw()
         project_id = get_project_id(project_definition.root)
 
+        if no_restart and restart_only:
+            raise click.ClickException(
+                "Cannot set both --restart-only and --no-restart at the same time."
+            )
+
         if not session_name:
             session_name = str(
                 self.api_client.get_project_default_session_name_api_v2_projects_project_id_default_session_name_get(
@@ -304,32 +371,58 @@ class SessionController:
 
         session_exists = len(session_list) > 0
 
-        if not session_exists:
-            config_attribute_provided = True
-            if not config:
-                config_attribute_provided = False
-                config = project_definition.cluster_yaml()
-
-            if not os.path.exists(config):
-                if config_attribute_provided:
-                    raise ValueError("Config file {} not found".format(config))
-                else:
-                    raise ValueError(
-                        "No config param provided and default config file session-default.yaml not found. Please provide a config file using the --config attribute."
-                    )
-
+        # Old cluster config launch path.
         cluster_config = None
+        if build_id is None and compute_template_id is None:
+            if session_exists and not config:
+                # Get cluster config stored in DB from existing session
+                session = session_list[0]
+                resp = self.api_client.get_session_cluster_config_api_v2_sessions_session_id_cluster_config_get(
+                    session.id
+                )
+                cluster_config = yaml.safe_load(resp.result.config_with_defaults)
+            elif not session_exists and not config:
+                # Get cluster config stored in DB from project
+                resp = self.api_client.get_project_latest_cluster_config_api_v2_projects_project_id_latest_cluster_config_get(
+                    project_id=project_id
+                )
+                cluster_config = yaml.safe_load(resp.result.config_with_defaults)
+            else:
+                # Get cluster config from `config` argument
+                config = str(config)
+                if not os.path.exists(config):
+                    raise ValueError("Config file {} not found".format(config))
+                with open(config) as f:
+                    cluster_config_filled = populate_session_args(f.read(), config)
+                    cluster_config = yaml.safe_load(cluster_config_filled)
+                validate_cluster_configuration(config, cluster_config, self.api_client)
 
-        if config:
-            with open(config) as f:
-                cluster_config_filled = populate_session_args(f.read(), config)
-                cluster_config = yaml.safe_load(cluster_config_filled)
+            cluster_config = _configure_min_max_workers(
+                cluster_config, min_workers, max_workers
+            )
 
-            validate_cluster_configuration(config, cluster_config, self.api_client)
+        # Build and compute template launch path.
+        else:
+            # Most of the logic is handled in the backend;
+            # here, we just need to make sure any CLI options are compatible.
+            if min_workers is not None or max_workers is not None:
+                # These are invalid options for now;
+                # later, we can add logic to add them to the compute template
+                # when we figure out what the semantics of this option
+                # with differing "available_node_types" are.
+                raise click.ClickException(
+                    "Cannot set --min-workers or --max-workers in CLI when using compute templates; please set directly in the compute template."
+                )
 
         cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_id, cloud_name)
         assert cloud_id is not None, "Failed to get cloud."
 
+        # We could also set the default idle_timeout in the function signature,
+        # but that would break backwards compatibility with older versions of
+        # the product which do have an "--idle-timeout" field
+        if idle_timeout is None:
+            idle_timeout = 120
+
         with format_api_exception(ApiException):
             up_response = self.api_client.start_empty_session_api_v2_sessions_start_empty_session_post(
                 session_up_options=SessionUpOptions(
@@ -338,29 +431,36 @@ class SessionController:
                     cluster_config={"config": json.dumps(cluster_config)}
                     if cluster_config
                     else None,
+                    build_id=build_id,
+                    compute_template_id=compute_template_id,
                     cloud_id=cloud_id,
+                    idle_timeout=idle_timeout,
+                    restart_only=restart_only,
+                    no_restart=no_restart,
                 )
             ).result
         session_id = up_response.session_id
         second_update_required = up_response.second_update_required
-        wait_for_session_start(project_id, session_name, self.api_client)
+        wait_for_head_node_start(project_id, session_name, self.api_client)
 
         # Push will sync project directory to working directory.
-        self.push(session_name, source=None, target=None, config=None, all_nodes=False)
+        self.push(
+            session_name, source=None, target=None, config=None, all_nodes=False,
+        )
 
-        if second_update_required:
-            with format_api_exception(ApiException):
-                self.api_client.setup_and_initialize_session_api_v2_sessions_session_id_setup_and_initialize_session_post(
-                    session_id=session_id,
-                    setup_initialize_session_options=SetupInitializeSessionOptions(
-                        project_id=project_id,
-                        cluster_config={"config": json.dumps(cluster_config)}
-                        if cluster_config
-                        else None,
-                    ),
-                )
+        with format_api_exception(ApiException):
+            self.api_client.setup_and_initialize_session_api_v2_sessions_session_id_setup_and_initialize_session_post(
+                session_id=session_id,
+                setup_initialize_session_options=SetupInitializeSessionOptions(
+                    project_id=project_id,
+                    cluster_config={"config": json.dumps(cluster_config)},
+                    restart_only=restart_only,
+                    no_restart=no_restart,
+                    no_update=not second_update_required,
+                ),
+            )
 
-            wait_for_session_start(project_id, session_name, self.api_client)
+        wait_for_session_start(project_id, session_name, self.api_client)
 
         url = get_endpoint(f"/projects/{project_id}")
         print(f"Session {session_name} started. View at {url}")
@@ -531,31 +631,23 @@ class SessionController:
             )
 
             source = canonicalize_remote_location(cluster_config, source, project_id)
-            with tempfile.NamedTemporaryFile(mode="w") as config_file:
-                json.dump(cluster_config, config_file)
-                config_file.flush()
 
-                with set_env(**aws_credentials):
-                    if source and target:
-                        rsync(
-                            config_file.name,
-                            source=source,
-                            target=target,
-                            override_cluster_name=None,
-                            down=True,
-                        )
-                    elif source or target:
-                        raise click.ClickException(
-                            "Source and target are not both specified. Please either specify both or neither."
-                        )
-                    else:
-                        rsync(
-                            config_file.name,
-                            source=source_directory,
-                            target=project_definition.root,
-                            override_cluster_name=None,
-                            down=True,
-                        )
+            with set_env(**aws_credentials):
+                if source and target:
+                    rsync(
+                        cluster_config, source=source, target=target, down=True,
+                    )
+                elif source or target:
+                    raise click.ClickException(
+                        "Source and target are not both specified. Please either specify both or neither."
+                    )
+                else:
+                    rsync(
+                        cluster_config,
+                        source=source_directory,
+                        target=project_definition.root,
+                        down=True,
+                    )
 
             if config:
                 session = get_project_session(project_id, session_name, self.api_client)
@@ -593,33 +685,19 @@ class SessionController:
             cluster_config
         )
 
-        with tempfile.NamedTemporaryFile(mode="w") as config_file:
-            json.dump(cluster_config, config_file)
-            config_file.flush()
-
-            with set_env(**aws_credentials):
-                if source and target:
-                    rsync(
-                        config_file.name,
-                        source=source,
-                        target=target,
-                        override_cluster_name=None,
-                        down=False,
-                        all_nodes=all_nodes,
-                    )
-                elif source or target:
-                    raise click.ClickException(
-                        "Source and target are not both specified. Please either specify both or neither."
-                    )
-                else:
-                    rsync(
-                        config_file.name,
-                        source=None,
-                        target=None,
-                        override_cluster_name=None,
-                        down=False,
-                        all_nodes=all_nodes,
-                    )
+        with set_env(**aws_credentials):
+            if source and target:
+                rsync(
+                    cluster_config, source=source, target=target, down=False,
+                )
+            elif source or target:
+                raise click.ClickException(
+                    "Source and target are not both specified. Please either specify both or neither."
+                )
+            else:
+                rsync(
+                    cluster_config, source=None, target=None, down=False,
+                )
 
         if config:
             validate_cluster_configuration(config, api_instance=self.api_client)
@@ -627,6 +705,8 @@ class SessionController:
             self.up(
                 session_name=session_name,
                 config=None,
+                build_id=None,
+                compute_template_id=None,
                 min_workers=None,
                 max_workers=None,
                 no_restart=False,
@@ -634,6 +714,7 @@ class SessionController:
                 disable_sync=True,
                 cloud_id=session.cloud_id,
                 cloud_name=None,
+                idle_timeout=None,
                 yes=True,
             )
         url = get_endpoint(f"/projects/{project_id}")
