@@ -363,6 +363,58 @@ def wait_for_session_start(
         retry_count += 1
 
 
+def wait_for_head_node_start(
+    project_id: str, session_name: str, api_client: Optional[DefaultApi] = None
+) -> str:
+    print(f"Waiting for head node of session {session_name} to start.")
+    if api_client is None:
+        api_client = get_api_client()
+
+    retry_count = 0
+    while True:
+        sessions = api_client.list_sessions_api_v2_sessions_get(
+            project_id=project_id, name=session_name, active_only=False
+        ).results
+
+        if len(sessions) > 0:
+            session = sessions[0]
+
+            if (
+                session.state == "Running" and session.pending_state is None
+            ) or session.state == "AwaitingFileMounts":
+                return cast(str, session.id)
+
+            # Check for start up errors
+            if (
+                session.state_data
+                and session.state_data.startup
+                and session.state_data.startup.startup_error
+            ):
+                raise click.ClickException(
+                    f"Error while starting session {session_name}: {session.state_data.startup.startup_error}"
+                )
+            elif session.state and "Errored" in session.state:
+                raise click.ClickException(
+                    f"Error while starting session {session_name}: Session startup failed due to an error."
+                )
+            elif (
+                retry_count >= 2
+                and session.state
+                and session.state in {"Terminated", "Stopped"}
+            ):
+                # Session is created in Terminated state; Ignore the first few state checks.
+                raise click.ClickException(
+                    f"Error while starting session {session_name}: Session is still in stopped/terminated state."
+                )
+        else:
+            raise click.ClickException(
+                f"Error while starting session {session_name}: Session doesn't exist."
+            )
+
+        time.sleep(5)
+        retry_count += 1
+
+
 def install_anyscale_hooks(cluster_config: Dict[str, Any]) -> None:
     "Add anyscale hooks into the cluster configuration."
 
@@ -516,21 +568,21 @@ def canonicalize_remote_location(
     cluster_config: Dict[str, Any], remote_location: Optional[str], project_id: str
 ) -> Optional[str]:
     """Returns remote_location, but changes it from being based
-    in "~/" or "/root/" to match working_dir
+    in "~/", "/root/" or "/ray" to match working_dir
     """
     # Include the /root path to ensure that absolute paths also work
     # This is because of an implementation detail in OSS Ray's rsync
     if bool(get_container_name(cluster_config)) and bool(remote_location):
         remote_location = str(remote_location)
         working_dir = get_working_dir(cluster_config, project_id)
+        for possible_name in ["root", "ray"]:
+            full_name = f"/{possible_name}/"
+            # TODO(ilr) upstream this to OSS Ray
+            if working_dir.startswith("~/") and remote_location.startswith(full_name):
+                return remote_location.replace(full_name, "~/", 1)
 
-        # TODO(ilr) upstream this to OSS Ray
-        # TODO(ilr) move away from hardcoded /root
-        if working_dir.startswith("~/") and remote_location.startswith("/root/"):
-            return remote_location.replace("/root/", "~/", 1)
-
-        if working_dir.startswith("/root/") and remote_location.startswith("~/"):
-            return remote_location.replace("~/", "/root/", 1)
+            if working_dir.startswith(full_name) and remote_location.startswith("~/"):
+                return remote_location.replace("~/", full_name, 1)
 
     return remote_location
 
