@@ -6,6 +6,8 @@ import aioredis
 from terminado import NamedTermManager, TermSocket
 import tornado.web
 
+from anyscale.webterminal.utils import write_zshrc
+
 
 class AnyscaleTermManager(NamedTermManager):  # type: ignore
     def __init__(self, **kwargs: Any) -> None:
@@ -69,9 +71,10 @@ class NewTerminalHandler(tornado.web.RequestHandler):
         self.application.settings["term_manager"].new_named_terminal()
 
 
-def make_application(deploy_environment: str) -> tornado.web.Application:
-    term_manager = AnyscaleTermManager(shell_command=["bash"], max_terminals=100)
-
+def make_application(deploy_environment: str, cwd: str) -> tornado.web.Application:
+    term_manager = AnyscaleTermManager(
+        shell_command=["bash"], max_terminals=100, term_settings={"cwd": cwd}
+    )
     handlers: Any = [
         (
             r"/webterminal/_websocket/(\w+)",
@@ -115,9 +118,16 @@ async def initialize_debugger(application: Any) -> None:
         await asyncio.sleep(1.0)
 
 
-def main(deploy_environment: str, use_debugger: bool) -> None:
-    application = make_application(deploy_environment)
-
+def main(
+    deploy_environment: str,
+    use_debugger: bool,
+    cli_token: str,
+    host: str,
+    working_dir: str,
+) -> None:
+    # Write the .zshrc file before starting the server.
+    write_zshrc()
+    application = make_application(deploy_environment, working_dir)
     port = 8700
     application.listen(port, "localhost")
     print("Listening on localhost:{}".format(port))
