@@ -1,5 +1,7 @@
 from datetime import datetime, timezone
-from unittest.mock import Mock
+from typing import Any
+from unittest.mock import Mock, patch
+
 
 import pytest
 
@@ -71,7 +73,7 @@ def session_test_data() -> Session:
         tensorboard_available=False,
         project_id="project_id",
         state="Running",
-        last_activity_at=datetime.now(tz=timezone.utc),
+        idle_timeout=120,
     )
 
 
@@ -88,7 +90,7 @@ def session_start_error_test_data() -> Session:
         state_data=SessionStateData(
             startup=SessionStartingUpData(startup_error="start up error")
         ),
-        last_activity_at=datetime.now(tz=timezone.utc),
+        idle_timeout=120,
     )
 
 
@@ -102,7 +104,7 @@ def session_terminated_test_data() -> Session:
         tensorboard_available=False,
         project_id="project_id",
         state="Terminated",
-        last_activity_at=datetime.now(tz=timezone.utc),
+        idle_timeout=120,
     )
 
 
@@ -125,3 +127,11 @@ def command_id_test_data() -> ExecuteCommandResponse:
         directory_name="dir_name",
         dns_address="session.anyscaleuserdata-dev.com",
     )
+
+
+@pytest.fixture(scope="session", autouse=True)
+def default_session_fixture(
+    request: "_pytest.fixtures.SubRequest",  # type: ignore # noqa: F821
+) -> Any:
+    with patch("time.sleep", Mock()):
+        yield
