@@ -141,8 +141,8 @@ def test_list_sessions_table(
         )
         expected_rows = [
             f"Active project: {mock_project_definition.root}",
-            "ACTIVE    SESSION       STATUS        CREATED       SNAPSHOT",
-            f"Y         {session_test_data.name}  TASK_RUNNING  0 second ago  N/A",
+            "ACTIVE    SESSION       STATUS        CREATED",
+            f"Y         {session_test_data.name}  TASK_RUNNING  0 second ago",
         ]
         expected_output = "\n".join(expected_rows)
 
